package io.reisub.dreambot.cwintertodt.tasks;

import io.reisub.dreambot.util.Constants;
import io.reisub.dreambot.util.TaskNodeParent;
import io.reisub.dreambot.util.Util;
import org.dreambot.api.methods.container.impl.Inventory;
import org.dreambot.api.methods.widget.Widgets;
import org.dreambot.api.script.ScriptManager;
import org.dreambot.api.script.listener.ChatListener;
import org.dreambot.api.script.listener.InventoryListener;
import org.dreambot.api.wrappers.items.Item;
import org.dreambot.api.wrappers.widgets.WidgetChild;
import org.dreambot.api.wrappers.widgets.message.Message;

public class WintertodtTask extends TaskNodeParent {
    private int rootCount, kindlingCount, energy;
    private boolean burnBeforeRunningOutOfTime, interrupted, burning;

    public static WintertodtTask createWintertodtTask() {
        WintertodtTask task = new WintertodtTask();

        task.setChildren(
                new LightOrFix(task),
                new Chop(task),
                new Fletch(task),
                new Burn(task)
        );

        return task;
    }

    private WintertodtTask() {
        ScriptManager.getScriptManager().addListener(new ChatListener() {
            @Override
            public void onMessage(Message message) {
                String msg = message.getMessage();

                if (msg.startsWith(Constants.WINTERTODT_COLD) || msg.startsWith(Constants.WINTERTODT_FREEZING_ATTACK)) {
                    setInterrupted(true);
                }
            }
        });

        ScriptManager.getScriptManager().addListener(new InventoryListener() {
            @Override
            public void onItemChange(Item[] items) {
                int burnableItems = Inventory.count(Constants.BRUMA_KINDLING) + Inventory.count(Constants.BRUMA_ROOT);

                if (burnableItems >= getEnergy() && !isBurning()) {
                    setInterrupted(true);
                }
            }
        });
    }

    @Override
    public boolean accept() {
        rootCount = Inventory.count(Constants.BRUMA_ROOT);
        kindlingCount = Inventory.count(Constants.BRUMA_KINDLING);
        energy = getWintertodtEnergy();

        burnBeforeRunningOutOfTime = rootCount + kindlingCount >= energy;

        if (isBurning() && rootCount == 0 && kindlingCount == 0) {
            setBurning(false);
        }

        return Util.playerIsIdle() || interrupted;
    }

    private int getWintertodtEnergy() {
        WidgetChild child = Widgets.getWidgetChild(396, 21);

        if (child == null) return -1;

        String text = child.getText();
        return Integer.parseInt(text.replaceAll("[\\D]", ""));
    }

    public int getEnergy() {
        return energy;
    }

    public int getRootCount() {
        return rootCount;
    }

    public int getKindlingCount() {
        return kindlingCount;
    }

    public boolean isInterrupted() {
        return interrupted;
    }

    public void setInterrupted(boolean interrupted) {
        this.interrupted = interrupted;
    }

    public boolean isBurning() {
        return burning;
    }

    public void setBurning(boolean burning) {
        this.burning = burning;
    }

    public boolean shouldBurnBeforeRunningOutOfTime() {
        return burnBeforeRunningOutOfTime;
    }
}
